%*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~;
%* Macro Crosschk crosschecks variable name lists for unique variable names  ;
%* and reports duplicates, forcing program abort via the Result argument.    ;
%* The macro variable specified as DupNames will contain a list of offending,;
%* duplicate names. The macro variable specified as DupNmbrs will contain a  ;
%* list of element numbers of offending, duplicate names in the &VarList2.   ;
%*___________________________________________________________________________;

%* (C) Jim Groeneveld, Netherlands ;
%* 21 October 2005, vs. 0.0.0, 4 September 2012, vs. 0.0.1 ;
%* Parameter Result must point to an already existing macro variable;
%* Macro variable &MacName assumed existing;

%*------------------------------------------------------------------------------;
%* 4 September 2012, vs. 0.0.1, changes                                         ;
%* - %SCAN into %SYSFUNC(SCANQ to allow for SAS name literal variable names     ;
%* - changed &ERR into %Err0r without following colon                           ;
%* - checking for empty macro value: (&Value NE) into (%LENGTH(%SUPERQ(Value))) ;
%* - extending functionality by making execution dependent on the specification ;
%*   of Result, DupNames and DupNmbrs that all should be existing to be used.   ;
%*   &Result already may contain event counts, while &DupNames and &DupNmbrs    ;
%*   should be initialised empty.                                               ;
%*------------------------------------------------------------------------------;

%MACRO CrossChk ( Element=Variable name, LstName1=, VarList1=, LstName2=, VarList2=,
                  Result=, DupNames=, DupNmbrs=, MacName=&MacName );
  %LOCAL I J Element1 Element2;

  %IF (%LENGTH(%SUPERQ(DupNames))) %THEN %LET &DupNames = ;
  %IF (%LENGTH(%SUPERQ(DupNmbrs))) %THEN %LET &DupNmbrs = ;

  %LET I = 1;
  %LET Element1 = %SYSFUNC(SCANQ(&VarList1,&I,%STR( )));
  %DO %WHILE (%LENGTH(%SUPERQ(Element1)));
    %LET J = 1;
    %LET Element2 = %SYSFUNC(SCANQ(&VarList2,&J,%STR( )));
    %DO %WHILE (%LENGTH(%SUPERQ(Element2)));
      %IF (%UPCASE(&Element1) EQ %UPCASE(&Element2)) %THEN %DO;
        %IF (%LENGTH(%SUPERQ(Result))) %THEN %DO;
          %IF (%UPCASE(&Result) EQ ERRCOUNT) %THEN %DO;
            %PUT *** &MacName *** %Err0r in parameter specification: ***;
            %PUT *** &Element &Element1 occurs in both &LstName1 and &LstName2;
            %PUT *** This is illegal, specify differing names. Macro &MacName will abort now.;
          %END;
          %LET &Result = %EVAL ( &&&Result + 1);
        %END;
        %IF (%LENGTH(%SUPERQ(DupNames))) %THEN %LET &DupNames = &&&DupNames &Element2;
        %IF (%LENGTH(%SUPERQ(DupNmbrs))) %THEN %LET &DupNmbrs = &&&DupNmbrs &J;
      %END;
      %LET J = %EVAL ( &J + 1 );
      %LET Element2 = %SYSFUNC(SCANQ(&VarList2,&J,%STR( )));
    %END; %*=== Inner While;
    %LET I = %EVAL ( &I + 1 );
    %LET Element1 = %SYSFUNC(SCANQ(&VarList1,&I,%STR( )));
  %END; %*=== Outer While;

%*~~~~~~~~~~~~;
%MEND CrossChk;
%*____________;
